@@ -27,6 +27,14 @@
 #include <config.h>
 #endif
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#ifndef PATH_MAX
+#define PATH_MAX 4096
+#endif
+
 #include <libxfce4util/i18n.h>
 #include <libxfcegui4/libxfcegui4.h>
 #include <libxfcegui4/xfce_scaled_image.h>
@@ -43,9 +51,11 @@ typedef struct _DMPlugin {
 	GtkWidget *button;
 	GtkWidget *image;
 	XfceDesktopMenu *desktop_menu;
+	gboolean use_default_menu;
 	gchar *menu_file;
 	gchar *icon_file;
 	gboolean show_menu_icons;
+	gchar *button_title;
 	
 	GtkWidget *file_entry;
 	GtkWidget *file_fb;
@@ -55,6 +65,61 @@ typedef struct _DMPlugin {
 	GtkTooltips *tooltip;  /* needed? */
 } DMPlugin;
 
+
+
+#if GTK_CHECK_VERSION(2, 6, 0)
+/* util */
+GtkWidget *
+xfutil_custom_button_new(const gchar *text, const gchar *icon)
+{
+	GtkWidget *btn, *hbox, *img, *lbl;
+	
+	hbox = gtk_hbox_new(FALSE, 4);
+	gtk_widget_show(hbox);
+	
+	img = gtk_image_new_from_stock(icon, GTK_ICON_SIZE_BUTTON);
+	if(img) {
+		if(gtk_image_get_storage_type(GTK_IMAGE(img)) != GTK_IMAGE_EMPTY) {
+			gtk_widget_show(img);
+			gtk_box_pack_start(GTK_BOX(hbox), img, FALSE, FALSE, 0);
+		} else
+			gtk_widget_destroy(img);
+	}
+	
+	lbl = gtk_label_new_with_mnemonic(text);
+	gtk_widget_show(lbl);
+	gtk_box_pack_start(GTK_BOX(hbox), lbl, FALSE, FALSE, 0);
+	
+	btn = gtk_button_new();
+	gtk_container_add(GTK_CONTAINER(btn), hbox);
+	gtk_label_set_mnemonic_widget(GTK_LABEL(lbl), btn);
+	
+	return btn;
+}
+#endif
+
+static gchar *
+dmp_get_real_path(const gchar *raw_path)
+{
+	gchar *path;
+	
+	if(!raw_path)
+		return NULL;
+	
+	if(strstr(raw_path, "$XDG_CONFIG_DIRS/") == raw_path)
+		return xfce_resource_lookup(XFCE_RESOURCE_CONFIG, raw_path+17);
+	else if(strstr(raw_path, "$XDG_CONFIG_HOME/") == raw_path)
+		return xfce_resource_save_location(XFCE_RESOURCE_CONFIG, raw_path+17, FALSE);
+	else if(strstr(raw_path, "$XDG_DATA_DIRS/") == raw_path)
+		return xfce_resource_lookup(XFCE_RESOURCE_DATA, raw_path+15);
+	else if(strstr(raw_path, "$XDG_DATA_HOME/") == raw_path)
+		return xfce_resource_save_location(XFCE_RESOURCE_DATA, raw_path+15, FALSE);
+	else if(strstr(raw_path, "$XDG_CACHE_HOME/") == raw_path)
+		return xfce_resource_save_location(XFCE_RESOURCE_CACHE, raw_path+16, FALSE);
+	
+	return xfce_expand_variables(raw_path, NULL);
+}
+
 static void
 dmp_set_size(Control *c, int size)
 {
@@ -92,6 +157,13 @@ dmp_free(Control *c)
 	if(dmp->tooltip)
 		gtk_object_sink(GTK_OBJECT(dmp->tooltip));
 	
+	if(dmp->menu_file)
+		g_free(dmp->menu_file);
+	if(dmp->icon_file)
+		g_free(dmp->icon_file);
+	if(dmp->button_title)
+		g_free(dmp->button_title);
+	
 	g_free(dmp);
 }
 
@@ -169,7 +241,7 @@ dmp_popup(GtkWidget *w, gpointer user_data)
 	
 	if(!dmp->desktop_menu) {
 		g_critical("dmp->desktop_menu is NULL - module load failed?");
-        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), FALSE);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), FALSE);
 		return;
 	}
 	
@@ -178,12 +250,12 @@ dmp_popup(GtkWidget *w, gpointer user_data)
 
 	menu = xfce_desktop_menu_get_widget(dmp->desktop_menu);
 	if(menu) {
-        guint id;
-        
+		guint id;
+		
 		panel_register_open_menu(menu);
-        id = g_signal_connect(menu, "deactivate", 
-                G_CALLBACK(menu_deactivated), dmp);
-        g_object_set_data(G_OBJECT(menu), "sig_id", GUINT_TO_POINTER(id));
+		id = g_signal_connect(menu, "deactivate", 
+				G_CALLBACK(menu_deactivated), dmp);
+		g_object_set_data(G_OBJECT(menu), "sig_id", GUINT_TO_POINTER(id));
 		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, 
 				(GtkMenuPositionFunc)dmp_position_menu, dmp->button->parent, 
 				1, gtk_get_current_event_time());
@@ -195,6 +267,7 @@ static DMPlugin *
 dmp_new()
 {
 	DMPlugin *dmp = g_new0(DMPlugin, 1);
+	dmp->use_default_menu = TRUE;
 	
 	dmp->show_menu_icons = TRUE;  /* default */
 	dmp->tooltip = gtk_tooltips_new();
@@ -202,12 +275,14 @@ dmp_new()
 	dmp->button = gtk_toggle_button_new();
 	gtk_button_set_relief(GTK_BUTTON(dmp->button), GTK_RELIEF_NONE);
 	gtk_widget_show(dmp->button);
-	gtk_tooltips_set_tip(dmp->tooltip, dmp->button, _("Xfce Menu"), NULL);
+	if(!dmp->button_title)
+		dmp->button_title = g_strdup(_("Xfce Menu"));
+	gtk_tooltips_set_tip(dmp->tooltip, dmp->button, dmp->button_title, NULL);
 	
 	dmp->image = xfce_scaled_image_new();
 	gtk_widget_show(dmp->image);
 	gtk_container_add(GTK_CONTAINER(dmp->button), dmp->image);
-    
+	
 	dmp->desktop_menu = xfce_desktop_menu_new(NULL, TRUE);
 	if(dmp->desktop_menu)
 		xfce_desktop_menu_start_autoregen(dmp->desktop_menu, 10);
@@ -239,18 +314,49 @@ dmp_read_config(Control *control, xmlNodePtr node)
 	xmlChar *value;
 	DMPlugin *dmp = control->data;
 	GdkPixbuf *pix;
+	gboolean redo_desktop_menu = FALSE, migration__got_menu_bool = FALSE;
+	
+	value = xmlGetProp(node, (const xmlChar *)"use_default_menu");
+	if(value) {
+		migration__got_menu_bool = TRUE;
+		if(*value == '1') {
+			if(!dmp->use_default_menu)
+				redo_desktop_menu = TRUE;
+			dmp->use_default_menu = TRUE;
+		} else {
+			if(dmp->use_default_menu)
+				redo_desktop_menu = TRUE;
+			dmp->use_default_menu = FALSE;
+		}
+		xmlFree(value);
+	}
 	
 	value = xmlGetProp(node, (const xmlChar *)"menu_file");
 	if(value) {
-		if(dmp->desktop_menu)
-			xfce_desktop_menu_destroy(dmp->desktop_menu);
+		if(!migration__got_menu_bool)
+			dmp->use_default_menu = FALSE;
+		
+		if(!dmp->use_default_menu)
+			redo_desktop_menu = TRUE;
+		
 		if(dmp->menu_file)
 			g_free(dmp->menu_file);
-		dmp->desktop_menu = xfce_desktop_menu_new(value, TRUE);
-		dmp->menu_file = g_strdup(value);
-	} else {
+		dmp->menu_file = value;
+	} else
+		dmp->use_default_menu = TRUE;
+	
+	if(redo_desktop_menu) {
 		if(dmp->desktop_menu)
-			dmp->menu_file = g_strdup(xfce_desktop_menu_get_menu_file(dmp->desktop_menu));
+			xfce_desktop_menu_destroy(dmp->desktop_menu);
+		
+		if(dmp->use_default_menu)
+			dmp->desktop_menu = xfce_desktop_menu_new(NULL, TRUE);
+		else {
+			gchar *path;
+			path = dmp_get_real_path(dmp->menu_file);
+			dmp->desktop_menu = xfce_desktop_menu_new(path, TRUE);
+			g_free(path);
+		}
 	}
 	
 	value = xmlGetProp(node, (const xmlChar *)"icon_file");
@@ -285,6 +391,15 @@ dmp_read_config(Control *control, xmlNodePtr node)
 			xfce_desktop_menu_set_show_icons(dmp->desktop_menu, dmp->show_menu_icons);
 		xmlFree(value);
 	}
+	
+	value = xmlGetProp(node, (const xmlChar *)"button_title");
+	if(value) {
+		if(dmp->button_title)
+			g_free(dmp->button_title);
+		dmp->button_title = value;
+		if(dmp->tooltip && dmp->button)
+			gtk_tooltips_set_tip(dmp->tooltip, dmp->button, dmp->button_title, NULL);
+	}
 }
 
 static void
@@ -292,9 +407,11 @@ dmp_write_config(Control *control, xmlNodePtr node)
 {
 	DMPlugin *dmp = control->data;
 	
+	xmlSetProp(node, (const xmlChar *)"use_default_menu", dmp->use_default_menu ? "1" : "0");
 	xmlSetProp(node, (const xmlChar *)"menu_file", dmp->menu_file ? dmp->menu_file : "");
 	xmlSetProp(node, (const xmlChar *)"icon_file", dmp->icon_file ? dmp->icon_file : "");
 	xmlSetProp(node, (const xmlChar *)"show_menu_icons", dmp->show_menu_icons ? "1" : "0");
+	xmlSetProp(node, (const xmlChar *)"button_title", dmp->button_title ? dmp->button_title : "");
 }
 
 static gboolean
@@ -324,8 +441,11 @@ entry_focus_out_cb(GtkWidget *w, GdkEventFocus *evt, gpointer user_data)
 		if(dmp->desktop_menu) {
 			cur_file = xfce_desktop_menu_get_menu_file(dmp->desktop_menu);
 			if(strcmp(dmp->menu_file, cur_file)) {
+				gchar *path;
 				xfce_desktop_menu_destroy(dmp->desktop_menu);
-				dmp->desktop_menu = xfce_desktop_menu_new(dmp->menu_file, TRUE);
+				path = dmp_get_real_path(dmp->menu_file);
+				dmp->desktop_menu = xfce_desktop_menu_new(path, TRUE);
+				g_free(path);
 				if(!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dmp->icons_chk)))
 					xfce_desktop_menu_set_show_icons(dmp->desktop_menu, FALSE);
 			}
@@ -442,31 +562,153 @@ icon_chk_cb(GtkToggleButton *w, gpointer user_data)
 		xfce_desktop_menu_set_show_icons(dmp->desktop_menu, dmp->show_menu_icons);
 }
 
+static void
+dmp_use_desktop_menu_toggled_cb(GtkToggleButton *tb, gpointer user_data)
+{
+	DMPlugin *dmp = user_data;
+	
+	if(gtk_toggle_button_get_active(tb)) {
+		GtkWidget *hbox;
+		
+		dmp->use_default_menu = TRUE;
+		
+		hbox = g_object_get_data(G_OBJECT(tb), "dmp-child-hbox");
+		gtk_widget_set_sensitive(hbox, FALSE);
+		
+		if(dmp->desktop_menu)
+			xfce_desktop_menu_destroy(dmp->desktop_menu);
+		dmp->desktop_menu = xfce_desktop_menu_new(NULL, TRUE);
+	}
+}
+
+static void
+dmp_use_custom_menu_toggled_cb(GtkToggleButton *tb, gpointer user_data)
+{
+	DMPlugin *dmp = user_data;
+	
+	if(gtk_toggle_button_get_active(tb)) {
+		GtkWidget *hbox;
+		
+		dmp->use_default_menu = FALSE;
+		
+		hbox = g_object_get_data(G_OBJECT(tb), "dmp-child-hbox");
+		gtk_widget_set_sensitive(hbox, TRUE);
+		
+		if(dmp->menu_file) {
+			if(dmp->desktop_menu)
+				xfce_desktop_menu_destroy(dmp->desktop_menu);
+			dmp->desktop_menu = xfce_desktop_menu_new(dmp->menu_file, TRUE);
+		}
+	}
+}
+
+static gboolean
+dmp_button_title_focus_out_cb(GtkWidget *w, GdkEventFocus *evt,
+		gpointer user_data)
+{
+	DMPlugin *dmp = user_data;
+	
+	if(dmp->button_title)
+		g_free(dmp->button_title);
+	dmp->button_title = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
+	
+	gtk_tooltips_set_tip(dmp->tooltip, dmp->button, dmp->button_title, NULL);
+	
+	return FALSE;
+}
+
+static void
+dmp_edit_menu_clicked_cb(GtkWidget *w, gpointer user_data)
+{
+	DMPlugin *dmp = user_data;
+	GError *err = NULL;
+	const gchar *menu_file;
+	gchar cmd[PATH_MAX];
+	
+	g_return_if_fail(dmp && dmp->desktop_menu);
+	
+	menu_file = xfce_desktop_menu_get_menu_file(dmp->desktop_menu);
+	if(!menu_file)
+		return;
+	
+	g_snprintf(cmd, PATH_MAX, "%s/xfce4-menueditor \"%s\"", BINDIR, menu_file);
+	if(xfce_exec(cmd, FALSE, FALSE, NULL))
+		return;
+	
+	g_snprintf(cmd, PATH_MAX, "xfce4-menueditor \"%s\"", menu_file);
+	if(!xfce_exec(cmd, FALSE, FALSE, &err)) {
+		xfce_warn(_("Unable to launch xfce4-menueditor: %s"), err->message);
+		g_error_free(err);
+	}
+}
+
 static void
 dmp_create_options(Control *ctrl, GtkContainer *con, GtkWidget *done)
 {
 	DMPlugin *dmp = ctrl->data;
-	GtkWidget *vbox, *hbox;
-	GtkWidget *label, *image, *filebutton, *chk;
-	GtkSizeGroup *sg;
+	GtkWidget *topvbox, *vbox, *hbox, *frame, *spacer;
+	GtkWidget *label, *image, *filebutton, *chk, *radio, *entry, *btn;
 	
 	xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
 
-	vbox = gtk_vbox_new(FALSE, BORDER/2);
-	gtk_widget_show(vbox);
-	gtk_container_add(con, vbox);
+	topvbox = gtk_vbox_new(FALSE, BORDER/2);
+	gtk_widget_show(topvbox);
+	gtk_container_add(con, topvbox);
 	
-	hbox = gtk_hbox_new(FALSE, BORDER);
+	hbox = gtk_hbox_new(FALSE, BORDER/2);
+	gtk_container_set_border_width(GTK_CONTAINER(hbox), BORDER/2);
 	gtk_widget_show(hbox);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(topvbox), hbox, FALSE, FALSE, 0);
 	
-	sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
-	
-	label = gtk_label_new_with_mnemonic(_("_Menu file:"));
-	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+	label = gtk_label_new_with_mnemonic(_("Button _title:"));
 	gtk_widget_show(label);
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-	gtk_size_group_add_widget(sg, label);
+	
+	entry = gtk_entry_new();
+	if(dmp->button_title)
+		gtk_entry_set_text(GTK_ENTRY(entry), dmp->button_title);
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), entry);
+	gtk_widget_show(entry);
+	gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+	g_signal_connect(G_OBJECT(entry), "focus-out-event",
+			G_CALLBACK(dmp_button_title_focus_out_cb), dmp);	
+	
+	frame = xfce_framebox_new(_("Menu File"), TRUE);
+	gtk_widget_show(frame);
+	gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
+	
+	vbox = gtk_vbox_new(FALSE, BORDER/2);
+	gtk_widget_show(vbox);
+	xfce_framebox_add(XFCE_FRAMEBOX(frame), vbox);
+	
+	/* 2nd radio button's child hbox */
+	hbox = gtk_hbox_new(FALSE, BORDER/2);
+	gtk_widget_show(hbox);
+	
+	radio = gtk_radio_button_new_with_mnemonic(NULL, _("Use default _desktop menu file"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), dmp->use_default_menu);
+	gtk_widget_show(radio);
+	gtk_box_pack_start(GTK_BOX(vbox), radio, FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT(radio), "toggled",
+			G_CALLBACK(dmp_use_desktop_menu_toggled_cb), dmp);
+	g_object_set_data(G_OBJECT(radio), "dmp-child-hbox", hbox);
+	
+	radio = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(radio),
+			_("Use _custom menu file:"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio), !dmp->use_default_menu);
+	gtk_widget_show(radio);
+	gtk_box_pack_start(GTK_BOX(vbox), radio, FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT(radio), "toggled",
+			G_CALLBACK(dmp_use_custom_menu_toggled_cb), dmp);
+	g_object_set_data(G_OBJECT(radio), "dmp-child-hbox", hbox);
+	
+	/* now pack in the child hbox */
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	
+	spacer = gtk_alignment_new(0.5, 0.5, 1, 1);
+	gtk_widget_show(spacer);
+	gtk_box_pack_start(GTK_BOX(hbox), spacer, FALSE, FALSE, 0);
+	gtk_widget_set_size_request(spacer, 16, -1);
 	
 	dmp->file_entry = gtk_entry_new();
 	if(dmp->menu_file)
@@ -492,7 +734,36 @@ dmp_create_options(Control *ctrl, GtkContainer *con, GtkWidget *done)
 	g_signal_connect(G_OBJECT(filebutton), "clicked",
 			G_CALLBACK(filebutton_click_cb), dmp);
 	
-	hbox = gtk_hbox_new(FALSE, BORDER);
+	gtk_widget_set_sensitive(hbox, !dmp->use_default_menu);
+	
+	spacer = gtk_alignment_new(0.5, 0.5, 1, 1);
+	gtk_widget_show(spacer);
+	gtk_box_pack_start(GTK_BOX(vbox), spacer, FALSE, FALSE, 0);
+	gtk_widget_set_size_request(spacer, -1, 4);
+	
+	hbox = gtk_hbox_new(FALSE, BORDER/2);
+	gtk_widget_show(hbox);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	
+#if GTK_CHECK_VERSION(2, 6, 0)
+	btn = xfutil_custom_button_new(_("_Edit Menu"), GTK_STOCK_EDIT);
+#else
+	btn = gtk_button_new_with_mnemonic(_("_Edit Menu"));
+#endif
+	gtk_widget_show(btn);
+	gtk_box_pack_end(GTK_BOX(hbox), btn, FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT(btn), "clicked",
+			G_CALLBACK(dmp_edit_menu_clicked_cb), dmp);
+	
+	frame = xfce_framebox_new(_("Icons"), TRUE);
+	gtk_widget_show(frame);
+	gtk_box_pack_start(GTK_BOX(topvbox), frame, FALSE, FALSE, 0);
+	
+	vbox = gtk_vbox_new(FALSE, BORDER/2);
+	gtk_widget_show(vbox);
+	xfce_framebox_add(XFCE_FRAMEBOX(frame), vbox);
+	
+	hbox = gtk_hbox_new(FALSE, BORDER/2);
 	gtk_widget_show(hbox);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 	
@@ -500,7 +771,6 @@ dmp_create_options(Control *ctrl, GtkContainer *con, GtkWidget *done)
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_widget_show(label);
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-	gtk_size_group_add_widget(sg, label);
 	
 	dmp->icon_entry = gtk_entry_new();
 	if(dmp->icon_file)
@@ -532,6 +802,8 @@ G_MODULE_EXPORT void
 xfce_control_class_init(ControlClass *cc)
 {
 	cc->name = "xfce-menu";
+	
+	xfce_textdomain(GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
 	cc->caption = _("Xfce Menu");
 	
 	cc->create_control = (CreateControlFunc)dmp_create;
